#!/usr/share/ucs-test/runner pytest-3 -s -l -v
## -*- coding: utf-8 -*-
## desc: Check id broker settings and cmdline tool
## tags: [ucsschool-id-broker]
## packages: [id-broker-common]
## exposure: dangerous

import json
import subprocess

import pytest

import univention.testing.strings as uts
import univention.testing.ucsschool.ucs_test_school as utu
from univention.config_registry import ConfigRegistry
from univention.idbroker.lib import generate_pseudonym
from univention.idbroker.udm_host import get_settings_data_content
from univention.udm import UDM
from univention.udm.encoders import Base64Bzip2BinaryProperty


@pytest.fixture()
def ldap_base():
    return ConfigRegistry().load().get("ldap/base")


@pytest.fixture()
def mappings_settings_data_dn(ldap_base):
    return f"cn=IDBrokerServiceProviderMappings,cn=IDBrokerSettings,cn=univention,{ldap_base}"


@pytest.fixture()
def secrets_settings_data_dn(ldap_base):
    return f"cn=IDBrokerServiceProviderSecrets,cn=IDBrokerSettings,cn=univention,{ldap_base}"


@pytest.fixture()
def udm():
    # We can't use TestUDM because we modify the settings/data objects.
    return UDM.admin().version(2)


@pytest.fixture()
def preserve_settings_data(udm):
    settings = []

    def _preserve_settings_data(dn):
        obj = udm.obj_by_dn(dn)
        settings.append((dn, obj.props.data.raw))
        obj.props.data = Base64Bzip2BinaryProperty("data", raw_value=b"{}")
        obj.save()

    yield _preserve_settings_data
    for dn, raw_data in settings:
        obj = udm.obj_by_dn(dn)
        obj.props.data = Base64Bzip2BinaryProperty("data", raw_value=raw_data)
        obj.save()


@pytest.mark.parametrize(
    "setting,settings_dn",
    [
        ("secrets", "secrets_settings_data_dn"),
        ("mappings", "mappings_settings_data_dn"),
    ],
)
def test_add_remove_setting(udm, setting, settings_dn, preserve_settings_data, request):
    settings_dn = request.getfixturevalue(settings_dn)
    preserve_settings_data(settings_dn)

    key = uts.random_string()
    value = uts.random_string()
    expected_data = {}
    expected_data[key] = value

    subprocess.run(
        [
            "manage-service-providers",
            "addsetting",
            setting,
            "--key",
            key,
            "--value",
            value,
        ]
    )
    res = subprocess.run(
        ["manage-service-providers", "showsetting", setting],
        capture_output=True,
    )
    res = json.loads(res.stdout)
    assert expected_data == res

    ldap_data = udm.obj_by_dn(settings_dn)
    ldap_data = json.loads(ldap_data.props.data.raw)

    assert expected_data == ldap_data
    assert ldap_data == res

    expected_data = {}
    subprocess.run(
        [
            "manage-service-providers",
            "removesetting",
            setting,
            "--key",
            key,
        ]
    )
    res = subprocess.run(
        ["manage-service-providers", "showsetting", setting],
        capture_output=True,
    )
    res = json.loads(res.stdout)
    assert expected_data == res

    ldap_data = udm.obj_by_dn(settings_dn)
    ldap_data = json.loads(ldap_data.props.data.raw)

    assert expected_data == ldap_data
    assert ldap_data == res


def test_add_remove_service_provider(
    udm, mappings_settings_data_dn, secrets_settings_data_dn, preserve_settings_data
):
    users_mod = udm.get("users/user")
    groups_mod = udm.get("groups/group")
    ous_mod = udm.get("container/ou")
    preserve_settings_data(mappings_settings_data_dn)
    preserve_settings_data(secrets_settings_data_dn)
    provisioned_users = len(list(users_mod.search("(&(ucsschoolSourceUID=*)(ucsschoolRecordUID=*))")))
    provisioned_groups = len(list(groups_mod.search("(&(ucsschoolSourceUID=*)(ucsschoolRecordUID=*))")))
    provisioned_ous = len(list(ous_mod.search("(&(ucsschoolSourceUID=*)(ucsschoolRecordUID=*))")))
    assert (
        provisioned_user + provisioned_groups + provisioned_ous == 0
    ), "This test assumes an environment without provisioned users"

    school_auth = uts.random_string()
    with utu.UCSTestSchool() as schoolenv:
        ou_name, ou_dn = schoolenv.create_ou(name_edudc=schoolenv.ucr["hostname"])
        student_name, student_dn = schoolenv.create_user(ou_name)
        student = udm.obj_by_dn(student_dn)
        student.props.ucsschoolSourceUID = school_auth
        student.props.ucsschoolRecordUID = uts.random_string()
        student.save()
        workgroup_name, workgroup_dn = schoolenv.create_workgroup(ou_name)
        schoolclass_name, schoolclass_dn = schoolenv.create_school_class(ou_name)
        schoolclass = udm.obj_by_dn(schoolclass_dn)
        schoolclass.props.ucsschoolSourceUID = school_auth
        schoolclass.props.ucsschoolRecordUID = uts.random_string()
        schoolclass.save()
        ou = udm.obj_by_dn(ou_dn)
        ou.props.ucsschoolSourceUID = school_auth
        ou.props.ucsschoolRecordUID = uts.random_string()
        ou.save()

        provisioned_user = udm.obj_by_dn(student_dn)
        assert provisioned_user.props.idBrokerPseudonym0001 is None
        provisioned_workgroup = udm.obj_by_dn(workgroup_dn)
        assert provisioned_workgroup.props.idBrokerPseudonym0001 is None
        provisioned_class = udm.obj_by_dn(schoolclass_dn)
        assert provisioned_class.props.idBrokerPseudonym0001 is None
        provisioned_ou = udm.obj_by_dn(ou_dn)
        assert provisioned_ou.props.idBrokerPseudonym0001 is None

        mappings = get_settings_data_content(mappings_settings_data_dn)
        assert mappings == {}
        secrets = get_settings_data_content(secrets_settings_data_dn)
        assert secrets == {}

        service_provider = uts.random_string()
        secret = uts.random_string()
        subprocess.run(
            [
                "manage-service-providers",
                "addservice",
                "--name",
                service_provider,
                "--secret",
                secret,
            ]
        )

        provisioned_user = udm.obj_by_dn(student_dn)
        pseudonym = generate_pseudonym(secret, provisioned_user.props.ucsschoolRecordUID, school_auth)
        assert provisioned_user.props.idBrokerPseudonym0001 == pseudonym

        provisioned_schoolclass = udm.obj_by_dn(schoolclass_dn)
        schoolclass_authority = provisioned_schoolclass.props.ucsschoolSourceUID
        schoolclass_record_uid = provisioned_schoolclass.props.ucsschoolRecordUID
        pseudonym = generate_pseudonym(secret, schoolclass_record_uid, schoolclass_authority)
        assert provisioned_schoolclass.props.idBrokerPseudonym0001 == pseudonym

        provisioned_ou = udm.obj_by_dn(ou_dn)
        ou_authority = provisioned_ou.props.ucsschoolSourceUID
        ou_record_uid = provisioned_ou.props.ucsschoolRecordUID
        pseudonym = generate_pseudonym(secret, ou_record_uid, ou_authority)
        assert provisioned_ou.props.idBrokerPseudonym0001 == pseudonym

        # not for workgroups
        provisioned_workgroup = udm.obj_by_dn(workgroup_dn)
        assert provisioned_workgroup.props.idBrokerPseudonym0001 is None

        mappings = get_settings_data_content(mappings_settings_data_dn)
        assert mappings == {service_provider: "idBrokerPseudonym0001"}
        secrets = get_settings_data_content(secrets_settings_data_dn)
        assert secrets == {service_provider: secret}

        subprocess.run(
            [
                "manage-service-providers",
                "removeservice",
                "--name",
                service_provider,
            ]
        )
        provisioned_user = udm.obj_by_dn(student_dn)
        assert provisioned_user.props.idBrokerPseudonym0001 is None
        mappings = get_settings_data_content(mappings_settings_data_dn)
        assert mappings == {}
        secrets = get_settings_data_content(secrets_settings_data_dn)
        assert secrets == {}
